"""
This module adds :mod:`asyncio` support for reading samples from the device.

The main functionality can be found in the :meth:`~rtlsdr.rtlsdraio.RtlSdrAio.stream`
method of :class:`rtlsdr.rtlsdraio.RtlSdrAio`.

Example:
    .. code-block:: python

       import asyncio
       from rtlsdr import RtlSdr

       async def streaming():
           sdr = RtlSdr()

           async for samples in sdr.stream():
               # do something with samples
               # ...

           # to stop streaming:
           await sdr.stop()

           # done
           sdr.close()

       loop = asyncio.get_event_loop()
       loop.run_until_complete(streaming())

"""

import logging
try:
    import asyncio
    AIO_AVAILABLE = True
except ImportError:
    AIO_AVAILABLE = False
from .rtlsdr import RtlSdr


log = logging.getLogger(__name__)

_CLASS_TEMPLATE = """
class AsyncCallbackIter:
    '''Convert a callback-based legacy async function into one supporting asyncio
    and Python 3.5+

    The queued data can be iterated using ``async for``

    Arguments:
        func_start: A callable which should take a single callback that will be
            passed data. Will be run in a seperate thread in case it blocks.
        func_stop (optional): A callable to stop ``func_start`` from calling the
            callback. Will be run in a seperate thread in case it blocks.
        queue_size (:obj:`int`, optional): The maximum amount of data
            that will be buffered.
        loop (optional): The ``asyncio.event_loop`` to use. If not supplied,
            :func:`asyncio.get_event_loop` will be used.

    '''

    def __init__(self, func_start, func_stop=None, queue_size=20, *, loop=None):
        self.queue = asyncio.Queue(queue_size)
        self.loop = loop if loop else asyncio.get_event_loop()
        self.func_stop = func_stop
        self.func_start = func_start

        self.running = False

    async def add_to_queue(self, *args):
        '''Add items to the queue

        Arguments:
            *args: Arguments to be added

        This method is a :obj:`~asyncio.coroutine`
        '''
        try:
            self.queue.put_nowait(args)
        except asyncio.QueueFull:
            log.info('extra callback data lost')

    def _callback(self, *args):
        if not self.running:
            return
        asyncio.run_coroutine_threadsafe(self.add_to_queue(*args), self.loop)

    async def start(self):
        '''Start the execution

        The callback given by ``func_start`` will be called by
        :meth:`asyncio.AbstractEventLoop.run_in_executor` and will continue
        until :meth:`stop` is called.

        This method is a :obj:`~asyncio.coroutine`
        '''

        assert(not self.running)

        # start legacy async function
        future = self.loop.run_in_executor(None, self.func_start, self._callback)
        asyncio.ensure_future(future, loop=self.loop)
        self.executor_task = future
        self.running = True

    async def stop(self):
        '''Stop the running executor task

        If ``func_stop`` was supplied, it will be called after the queue has
        been exhausted.

        This method is a :obj:`~asyncio.coroutine`
        '''

        assert(self.running)

        self.running = False

        # send a signal to stop
        iter_stopped = False
        while not iter_stopped:
            try:
                self.queue.put_nowait((StopAsyncIteration(),))
                iter_stopped = True
            except asyncio.QueueFull:
                try:
                    self.queue.task_done()
                except ValueError:
                    pass
        if self.func_stop:
            # stop legacy async function
            await self.loop.run_in_executor(None, self.func_stop)
        await self.executor_task

    def __aiter__(self):
        return self

    async def __anext__(self):
        val = await self.queue.get()
        self.queue.task_done()

        if isinstance(val[0], StopAsyncIteration):
            raise StopAsyncIteration

        #return val if len(val) > 1 else val[0]

        # slight hack for rtlsdr to ignore context object
        return val[0]
"""


if AIO_AVAILABLE:
    try:
        exec('async def test_for_async(): pass')
        exec('def test_unpack_operators(a, *, b): pass')
    except SyntaxError:
        AIO_AVAILABLE = False

if AIO_AVAILABLE:
    exec(_CLASS_TEMPLATE, globals(), locals())

class RtlSdrAio(RtlSdr):
    DEFAULT_READ_SIZE = 128*1024

    def stream(self, num_samples_or_bytes=DEFAULT_READ_SIZE, format='samples', loop=None):
        """Start async streaming from SDR and return an async iterator (Python 3.5+).

        The :meth:`read_samples_async` method is called in an  :class:`~concurrent.futures.Excecutor`
        instance using :meth:`asyncio.AbstractEventLoop.run_in_executor`.

        The returned asynchronous iterable can then used to retrieve sample
        data using ``async for`` syntax.

        Calling the :meth:`~rtlsdr.rtlsdraio.RtlSdrAio.stop` method will stop
        the ``read_samples_async`` session and close the ``Excecutor`` task.

        Arguments:
            num_samples_or_bytes (int): The number of bytes/samples that will be
                returned each iteration
            format (:obj:`str`, optional): Specifies whether raw data ("bytes")
                or IQ samples ("samples") will be returned
            loop (optional): An asyncio event loop

        Returns:
            An ``asynchronous iterator`` to yield sample data
        """
        if format == 'samples':
            func_start = self.read_samples_async
        elif format == 'bytes':
            func_start = self.read_bytes_async
        else:
            raise ValueError('format "%s" not supported' % format)

        self.async_iter = AsyncCallbackIter(func_start=lambda cb: func_start(cb, num_samples_or_bytes),
                                            func_stop=self.cancel_read_async,
                                            loop=loop)
        asyncio.ensure_future(self.async_iter.start(), loop=loop)

        return self.async_iter

    def stop(self):
        """Stop async stream

        Stops the ``read_samples_async`` and ``Excecutor`` task created by
        :meth:`stream`.
        """
        return asyncio.ensure_future(self.async_iter.stop(), loop=self.async_iter.loop)



#    This file is part of pyrlsdr.
#    Copyright (C) 2013 by Roger <https://github.com/roger-/pyrtlsdr>
#
#    pyrlsdr is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    pyrlsdr is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with pyrlsdr.  If not, see <http://www.gnu.org/licenses/>.


import sys
import os
from ctypes import *
from ctypes.util import find_library

def load_librtlsdr():
    if sys.platform == "linux" and 'LD_LIBRARY_PATH' in os.environ.keys():
        ld_library_paths = [local_path for local_path in os.environ['LD_LIBRARY_PATH'].split(':') if local_path.strip()]
        driver_files = [local_path + '/librtlsdr.so' for local_path in ld_library_paths]
    else:
        driver_files = []
    driver_files += ['librtlsdr.so', 'rtlsdr/librtlsdr.so']
    driver_files += ['rtlsdr.dll', 'librtlsdr.so']
    driver_files += ['..//rtlsdr.dll', '..//librtlsdr.so']
    driver_files += ['rtlsdr//rtlsdr.dll', 'rtlsdr//librtlsdr.so']
    driver_files += [lambda : find_library('rtlsdr'), lambda : find_library('librtlsdr')]
    dll = None

    for driver in driver_files:
        if callable(driver):
            driver = driver()
        try:
            dll = CDLL(driver)
            break
        except:
            pass
    else:
        raise ImportError('Error loading librtlsdr. Make sure librtlsdr '\
                          '(and all of its dependencies) are in your path')

    return dll

librtlsdr = load_librtlsdr()

# we don't care about the rtlsdr_dev struct and it's allocated by librtlsdr, so
# we won't even bother filling it in
p_rtlsdr_dev = c_void_p

# async callbacks must be passed through this function
# typedef void(*rtlsdr_read_async_cb_t)(unsigned char *buf, uint32_t len, void *ctx);
rtlsdr_read_async_cb_t = CFUNCTYPE(None, POINTER(c_ubyte), c_int, py_object)

# uint32_t rtlsdr_get_device_count(void);
f = librtlsdr.rtlsdr_get_device_count
f.restype, f.argtypes = c_uint, []

# const char* rtlsdr_get_device_name(uint32_t index);
f = librtlsdr.rtlsdr_get_device_name
f.restype, f.argtypes = c_char_p, [c_uint]

# int rtlsdr_get_device_usb_strings(uint32_t index, char *manufact,
#                                   char *product, char *serial)
f = librtlsdr.rtlsdr_get_device_usb_strings
f.restype, f.argtypes = c_int, [c_uint,
                                POINTER(c_ubyte),
                                POINTER(c_ubyte),
                                POINTER(c_ubyte)]

# int rtlsdr_get_index_by_serial(const char *serial);
f = librtlsdr.rtlsdr_get_index_by_serial
f.restype, f.argtypes = c_int, [c_char_p]

# int rtlsdr_open(rtlsdr_dev_t **dev, uint32_t index);
f = librtlsdr.rtlsdr_open
f.restype, f.argtypes = c_int, [POINTER(p_rtlsdr_dev), c_uint]

# int rtlsdr_close(rtlsdr_dev_t *dev);
f = librtlsdr.rtlsdr_close
f.restype, f.argtypes = c_int, [p_rtlsdr_dev]

# /* configuration functions */

# int rtlsdr_set_center_freq(rtlsdr_dev_t *dev, uint32_t freq);
f = librtlsdr.rtlsdr_set_center_freq
f.restype, f.argtypes = c_int, [p_rtlsdr_dev, c_uint]

# int rtlsdr_get_center_freq(rtlsdr_dev_t *dev);
f = librtlsdr.rtlsdr_get_center_freq
f.restype, f.argtypes = c_uint, [p_rtlsdr_dev]

# int rtlsdr_set_freq_correction(rtlsdr_dev_t *dev, int ppm);
f = librtlsdr.rtlsdr_set_freq_correction
f.restype, f.argtypes = c_int, [p_rtlsdr_dev, c_int]

# int rtlsdr_get_freq_correction(rtlsdr_dev_t *dev);
f = librtlsdr.rtlsdr_get_freq_correction
f.restype, f.argtypes = c_int, [p_rtlsdr_dev]

# enum rtlsdr_tuner rtlsdr_get_tuner_type(rtlsdr_dev_t *dev);
f = librtlsdr.rtlsdr_get_tuner_type
f.restype, f.argtypes = c_int, [p_rtlsdr_dev]

# int rtlsdr_set_tuner_gain(rtlsdr_dev_t *dev, int gain);
f = librtlsdr.rtlsdr_set_tuner_gain
f.restype, f.argtypes = c_int, [p_rtlsdr_dev, c_int]

# int rtlsdr_get_tuner_gain(rtlsdr_dev_t *dev);
f = librtlsdr.rtlsdr_get_tuner_gain
f.restype, f.argtypes = c_int, [p_rtlsdr_dev]

# int rtlsdr_get_tuner_gains(rtlsdr_dev_t *dev, int *gains)
f = librtlsdr.rtlsdr_get_tuner_gains
f.restype, f.argtypes = c_int, [p_rtlsdr_dev, POINTER(c_int)]

# RTLSDR_API int rtlsdr_set_tuner_gain_mode(rtlsdr_dev_t *dev, int manual);
f = librtlsdr.rtlsdr_set_tuner_gain_mode
f.restype, f.argtypes = c_int, [p_rtlsdr_dev, c_int]

# RTLSDR_API int rtlsdr_set_agc_mode(rtlsdr_dev_t *dev, int on);
f = librtlsdr.rtlsdr_set_agc_mode
f.restype, f.argtypes = c_int, [p_rtlsdr_dev, c_int]

# RTLSDR_API  int rtlsdr_set_direct_sampling(rtlsdr_dev_t *dev, int on)
f = librtlsdr.rtlsdr_set_direct_sampling
f.restype, f.argtypes = c_int, [p_rtlsdr_dev, c_int]


# int rtlsdr_set_sample_rate(rtlsdr_dev_t *dev, uint32_t rate);
f = librtlsdr.rtlsdr_set_sample_rate
f.restype, f.argtypes = c_int, [p_rtlsdr_dev, c_uint]

# int rtlsdr_get_sample_rate(rtlsdr_dev_t *dev);
f = librtlsdr.rtlsdr_get_sample_rate
f.restype, f.argtypes = c_uint, [p_rtlsdr_dev]

# int rtlsdr_set_and_get_tuner_bandwidth(rtlsdr_dev_t *dev, uint32_t bw, uint32_t *applied_bw, int apply_bw );
try:
    f = librtlsdr.rtlsdr_set_and_get_tuner_bandwidth
    f.restype, f.argtypes = c_uint, [p_rtlsdr_dev, c_uint32, POINTER(c_uint32), c_int]
    tuner_bandwidth_supported = True
except AttributeError:
    tuner_bandwidth_supported = False

# int rtlsdr_set_tuner_bandwidth(rtlsdr_dev_t *dev, uint32_t bw);
try:
    f = librtlsdr.rtlsdr_set_tuner_bandwidth
    f.restype, f.argtypes = c_uint, [p_rtlsdr_dev, c_uint]
    tuner_set_bandwidth_supported = True
except AttributeError:
    tuner_set_bandwidth_supported = False

#/* streaming functions */

# int rtlsdr_reset_buffer(rtlsdr_dev_t *dev);
f = librtlsdr.rtlsdr_reset_buffer
f.restype, f.argtypes = c_int, [p_rtlsdr_dev]

# int rtlsdr_read_sync(rtlsdr_dev_t *dev, void *buf, int len, int *n_read);
f = librtlsdr.rtlsdr_read_sync
f.restype, f.argtypes = c_int, [p_rtlsdr_dev, c_void_p, c_int, POINTER(c_int)]

# int rtlsdr_wait_async(rtlsdr_dev_t *dev, rtlsdr_read_async_cb_t cb, void *ctx);
f = librtlsdr.rtlsdr_wait_async
f.restype, f.argtypes = c_int, [p_rtlsdr_dev, POINTER(rtlsdr_read_async_cb_t), py_object]

#int rtlsdr_read_async(rtlsdr_dev_t *dev,
#				 rtlsdr_read_async_cb_t cb,
#				 void *ctx,
#				 uint32_t buf_num,
#				 uint32_t buf_len);
f = librtlsdr.rtlsdr_read_async
f.restype, f.argtypes = c_int, [p_rtlsdr_dev, rtlsdr_read_async_cb_t, py_object, c_uint, c_uint]

# int rtlsdr_cancel_async(rtlsdr_dev_t *dev);
f = librtlsdr.rtlsdr_cancel_async
f.restype, f.argtypes = c_int, [p_rtlsdr_dev]

# RTLSDR_API int rtlsdr_set_xtal_freq(rtlsdr_dev_t *dev, uint32_t rtl_freq,
#				    uint32_t tuner_freq);
f = librtlsdr.rtlsdr_set_xtal_freq
f.restype, f.argtypes = c_int, [p_rtlsdr_dev, c_uint, c_uint]

# RTLSDR_API int rtlsdr_get_xtal_freq(rtlsdr_dev_t *dev, uint32_t *rtl_freq,
#				    uint32_t *tuner_freq);
f = librtlsdr.rtlsdr_get_xtal_freq
f.restype, f.argtypes = c_int, [p_rtlsdr_dev, POINTER(c_uint), POINTER(c_uint)]

# RTLSDR_API int rtlsdr_set_testmode(rtlsdr_dev_t *dev, int on);
f = librtlsdr.rtlsdr_set_testmode
f.restype, f.argtypes = c_int, [p_rtlsdr_dev, c_int]

__all__  = ['librtlsdr', 'p_rtlsdr_dev', 'rtlsdr_read_async_cb_t']



from __future__ import division
import sys
import time
import select
import socket
import struct
import errno
import traceback
import json

try:
    from itertools import izip
except ImportError:
    izip = zip

has_numpy = True
try:
    import numpy as np
except ImportError:
    has_numpy = False

PY2 = sys.version_info[0] == 2

DEFAULT_READ_SIZE = 1024
MAX_BUFFER_SIZE = 4096
RECEIVE_TIMEOUT = 20


class CommunicationError(Exception):
    def __init__(self, msg, source_exc=None):
        self.msg = msg
        self.source_exc = source_exc

    def __str__(self):
        s = self.msg
        if self.source_exc is not None:
            s = 'SOURCE EXCEPTION:\n%s\n\n%s' % (traceback.format_exc(), s)
        return s


class RtlSdrTcpBase(object):
    """Base class for all ``rtlsdrtcp`` functionality

    Arguments:
        device_index (:obj:`int`, optional):
        test_mode_enabled (:obj:`bool`, optional):
        hostname (:obj:`str`, optional):
        port (:obj:`int`, optional):
    """
    # Use port 1235 as default since rtl_tcp uses 1234
    DEFAULT_PORT = 1235

    def __init__(self, device_index=0, test_mode_enabled=False,
                 hostname='127.0.0.1', port=None):
        self.device_index = device_index
        self.test_mode_enabled = test_mode_enabled
        self.hostname = hostname
        self.port = port
        if self.port is None:
            self.port = self.DEFAULT_PORT
        self.device_ready = False
        self.server_thread = None

    def packed_bytes_to_iq(self, bytes):
        """A direct copy of :meth:`rtlsdr.BaseRtlSdr.packed_bytes_to_iq`
        """

        if has_numpy:
            # use NumPy array
            iq = np.empty(len(bytes)//2, 'complex')
            iq.real, iq.imag = bytes[::2], bytes[1::2]
            iq /= (255/2)
            iq -= (1 + 1j)
        else:
            # use normal list
            iq = [complex(i/(255/2) - 1, q/(255/2) - 1) for i, q in izip(bytes[::2], bytes[1::2])]

        return iq


API_METHODS = (
    'get_center_freq', 'set_center_freq',
    'get_sample_rate', 'set_sample_rate',
    'get_bandwidth', 'set_bandwidth',
    'get_gain', 'set_gain',
    'get_freq_correction', 'set_freq_correction',
    'get_gains',
    'get_tuner_type',
    'set_direct_sampling',
    'read_bytes',
    'read_samples',
)
API_DESCRIPTORS = {
    'center_freq', 'fc',
    'sample_rate', 'rs',
    'bandwidth',
    'gain',
    'freq_correction',
}


class MessageBase(object):
    """Base class for messages sent between clients and servers.

    Hanldes serialization/deserialization and communication with
    socket type objects.

    Attributes:
        timestamp (float): Timestamp given from :func:`time.time`
        header (dict): A ``dict`` containing message type and payload information
        data: The payload containing either the request or response data
    """

    def __init__(self, **kwargs):
        self.timestamp = kwargs.get('timestamp')
        self.header = self.get_header(**kwargs)
        self.data = self.get_data(**kwargs)

    @staticmethod
    def _send(sock, data):
        if not PY2 and isinstance(data, str):
            data = data.encode()
        r, w, e = select.select([], [sock], [], .5)
        if sock not in w:
            raise CommunicationError('socket %r not ready for write' % (sock))
        return sock.send(data)

    @staticmethod
    def _recv(sock):
        start_ts = time.time()
        r, w, e = select.select([sock], [], [], RECEIVE_TIMEOUT)
        if not len(r):
            now = time.time()
            raise CommunicationError('No response from peer after %s seconds' % (now - start_ts))
        if sock not in r:
            raise CommunicationError('socket %r not ready for read' % (sock))
        return sock.recv(MAX_BUFFER_SIZE)

    @classmethod
    def from_remote(cls, sock):
        """Reads data from the socket and parses an instance of :class:`MessageBase`

        Arguments:
            sock: The :class:`~socket.socket` object to read from

        """
        header = cls._recv(sock)
        if not PY2:
            header = header.decode()
        kwargs = json.loads(header)
        if kwargs.get('ACK'):
            cls = AckMessage
        return cls(**kwargs)

    def get_header(self, **kwargs):
        """Builds the header data for the message

        The :attr:`timestamp` is added to the header if not already present.

        Returns:
            dict:
        """
        d = {}
        ts = kwargs.get('timestamp')
        if ts is None:
            ts = time.time()
        d['timestamp'] = ts
        return d

    def get_data(self, **kwargs):
        return kwargs.get('data', kwargs.get('header', {}).get('data'))

    def send_message(self, sock):
        """Serializes and sends the message

        Arguments:
            sock: The :class:`~socket.socket` object to write to

        """
        header, data = self._serialize()
        self._send(sock, header)

    def get_response(self, sock):
        """Waits for a specific response message

        The message class returned from :meth:`get_response_class` is used
        to parse the message (called from :meth:`from_remote`)

        Arguments:
            sock: The :class:`~socket.socket` object to read from
        """
        cls = self.get_response_class()
        return cls.from_remote(sock)

    def get_ack_response(self, sock):
        return AckMessage.from_remote(sock)

    def _serialize(self):
        """Serializes the message header and data
        """
        struct_fmt = self.header.get('struct_fmt')
        if struct_fmt is not None:
            return json.dumps(self.header), self.data
        data = self.header.copy()
        data.setdefault('data', self.data)
        return json.dumps(data), None



class AckMessage(MessageBase):
    """Simple message type meant for ACKnolegemnt of message receipt
    """
    def get_header(self, **kwargs):
        d = super(AckMessage, self).get_header(**kwargs)
        d['ACK'] = True
        d['ok'] = kwargs.get('ok', True)
        return d


class ServerMessage(MessageBase):

    @classmethod
    def from_remote(cls, sock):
        """Reads data for the socket buffer and reconstructs the appropriate
        message that was sent by the other end.

        This method is used by clients to reconstruct ServerMessage objects
        and if necessary, use multiple read calls to get the entire message
        (if the message size is greater than the buffer length)

        """
        header = cls._recv(sock)
        if not PY2:
            header = header.decode()
        kwargs = json.loads(header)
        struct_fmt = kwargs.get('struct_fmt')
        if struct_fmt is not None:
            struct_fmt = str(struct_fmt)
            data_len = struct.calcsize(struct_fmt)
        else:
            return cls(**kwargs)
        ack_msg = AckMessage()
        ack_msg.send_message(sock)
        recv = None
        while data_len > 0:
            _recv = cls._recv(sock)
            if recv is None:
                recv = _recv
            else:
                recv += _recv
            data_len -= len(_recv)
        kwargs['data'] = struct.unpack(struct_fmt, recv)
        return cls(**kwargs)

    def send_message(self, sock):
        """Sends the message data to clients.

        If necessary, uses multiple calls to send to ensure all data has
        actually been sent through the socket objects's buffer.

        """
        header, data = self._serialize()

        self._send(sock, header)
        if isinstance(self.data, dict):
            struct_fmt = self.data.get('struct_fmt')
        else:
            struct_fmt = None
        if struct_fmt is not None:
            struct_fmt = str(struct_fmt)
            data = self.data['data']
            data_len = struct.calcsize(struct_fmt)
            ack = self.get_ack_response(sock)
            if not ack.header.get('ok'):
                raise CommunicationError('No ACK received')
            while data_len > 0:
                sent = self._send(sock, data)
                data_len -= sent
                data = data[sent:]

    def get_header(self, **kwargs):
        d = super(ServerMessage, self).get_header(**kwargs)
        d['success'] = kwargs.get('success', True)
        client_message = kwargs.get('client_message')
        if client_message is not None:
            d['request'] = client_message.header
        else:
            d['request'] = kwargs.get('request')
        return d

    def get_data(self, **kwargs):
        d = super(ServerMessage, self).get_data(**kwargs)
        if isinstance(d, dict) and 'struct_fmt' in d:
            self.header['struct_fmt'] = d['struct_fmt']
        return d

    def get_response_class(self):
        return AckMessage


class ClientMessage(MessageBase):
    def send_message(self, sock):
        super(ClientMessage, self).send_message(sock)
        return self.get_response(sock)

    def get_header(self, **kwargs):
        d = super(ClientMessage, self).get_header(**kwargs)
        keys = ['type', 'name']
        d.update({k: kwargs.get(k) for k in keys})
        return d

    def get_response_class(self):
        return ServerMessage



#    This file is part of pyrlsdr.
#    Copyright (C) 2013 by Roger <https://github.com/roger-/pyrtlsdr>
#
#    pyrlsdr is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    pyrlsdr is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with pyrlsdr.  If not, see <http://www.gnu.org/licenses/>.


from __future__ import division, print_function
from functools import wraps
import time

def limit_time(max_seconds):
    """Decorator to cancel async reads after a specified time period.

    Arguments:
        max_seconds: Number of seconds to wait before cancelling

    Examples:
        Stop reading after 10 seconds:
            >>> @limit_time(10)
            >>> def read_callback(data, context):
            >>>     print('signal mean:', sum(data)/len(data))
            >>> sdr = RtlSdr()
            >>> sdr.read_samples_async(read_callback)

    Notes:
        The context in either :meth:`~rtlsdr.RtlSdr.read_bytes_async`
        or :meth:`~rtlsdr.RtlSdr.read_samples_async` is relied upon and must
        use the default value (the :class:`~rtlsdr.RtlSdr` instance)
    """
    def decorator(f):
        f._start_time = None

        @wraps(f)
        def wrapper(buffer, rtlsdr_obj):
            if f._start_time is None:
                f._start_time = time.time()

            elapsed = time.time() - f._start_time
            if elapsed < max_seconds:
                return f(buffer, rtlsdr_obj)

            rtlsdr_obj.cancel_read_async()

            return

        return wrapper
    return decorator


def limit_calls(max_calls):
    """Decorator to cancel async reads after the given number of calls.

    Arguments:
        max_calls (int): Number of calls to wait for before cancelling

    Examples:
        Stop reading after 10 calls:
            >>> @limit_calls(10)
            >>> def read_callback(data, context):
            >>>     print('signal mean:', sum(data)/len(data))
            >>> sdr = RtlSdr()
            >>> sdr.read_samples_async(read_callback)

    Notes:
        See notes in :func:`limit_time`
    """
    def decorator(f):
        f._num_calls = 0

        @wraps(f)
        def wrapper(buffer, rtlsdr_obj):
            f._num_calls += 1

            if f._num_calls <= max_calls:
                return f(buffer, rtlsdr_obj)

            rtlsdr_obj.cancel_read_async()

            return

        return wrapper
    return decorator




import socket

from .base import (
    CommunicationError,
    RtlSdrTcpBase,
    ClientMessage,
    ServerMessage,
    AckMessage,
    DEFAULT_READ_SIZE,
)

class RtlSdrTcpClient(RtlSdrTcpBase):

    """Client object that connects to a remote server.

    Exposes most of the methods and descriptors that are available in the
    RtlSdr class in a transparent manner allowing an interface that is nearly
    identical to the core API.

    """

    def __init__(self, device_index=0, test_mode_enabled=False,
                 hostname='127.0.0.1', port=None):
        super(RtlSdrTcpClient, self).__init__(device_index, test_mode_enabled,
                                              hostname, port)
        self.open()

    def open(self, *args):
        self._socket = None
        self._keep_alive = False
        self.device_opened = True

    def close(self):
        self.device_opened = False

    def _build_socket(self):
        s = getattr(self, '_socket', None)
        if s is None:
            s = self._socket = socket.socket(socket.AF_INET,
                                             socket.SOCK_STREAM)
            s.connect((self.hostname, self.port))
        return s

    def _close_socket(self):
        if self._keep_alive:
            return
        s = getattr(self, '_socket', None)
        if s is None:
            return
        print('client closing socket')
        s.close()
        self._socket = None

    def _communicate(self, tx_message):
        s = self._build_socket()
        resp = tx_message.send_message(s)
        if isinstance(resp, ServerMessage):
            if not resp.header.get('success'):
                msg = 'server was unsuccessful. msg=%s' % (tx_message.header)
                raise CommunicationError(msg)
            resp_data = resp.data
        elif isinstance(resp, AckMessage):
            if not resp.header.get('ok'):
                raise CommunicationError('ACK message recieved as "NAK"')
            resp_data = None
        self._close_socket()
        return resp_data

    def _communicate_method(self, method_name, arg=None):
        msg = ClientMessage(type='method', name=method_name, data=arg)
        return self._communicate(msg)

    def _communicate_descriptor_get(self, prop_name):
        msg = ClientMessage(type='prop_get', name=prop_name)
        return self._communicate(msg)

    def _communicate_descriptor_set(self, prop_name, value):
        msg = ClientMessage(type='prop_set', name=prop_name, data=value)
        return self._communicate(msg)

    def get_center_freq(self):
        return self._communicate_descriptor_get('fc')

    def set_center_freq(self, value):
        self._communicate_descriptor_set('fc', value)

    def get_sample_rate(self):
        return self._communicate_descriptor_get('rs')

    def set_sample_rate(self, value):
        self._communicate_descriptor_set('rs', value)

    def get_bandwidth(self):
        return self._communicate_descriptor_get('bandwidth')

    def set_bandwidth(self, value):
        self._communicate_descriptor_set('bandwidth', value)

    def get_gain(self):
        return self._communicate_descriptor_get('gain')

    def set_gain(self, value):
        self._communicate_descriptor_set('gain', value)

    def get_freq_correction(self):
        return self._communicate_descriptor_get('freq_correction')

    def set_freq_correction(self, value):
        self._communicate_descriptor_set('freq_correction', value)

    def get_gains(self):
        return self._communicate_method('get_gains')

    def get_tuner_type(self):
        return self._communicate_method('get_tuner_type')

    def set_direct_sampling(self, value):
        self._communicate_method('set_direct_sampling', value)

    def read_bytes(self, num_bytes=DEFAULT_READ_SIZE):
        return self._communicate_method('read_bytes', num_bytes)

    def read_samples(self, num_samples=DEFAULT_READ_SIZE):
        raw_data = self._communicate_method('read_samples', num_samples)
        iq = self.packed_bytes_to_iq(raw_data)
        return iq

    def read_samples_async(self, *args):
        raise NotImplementedError('Async read not available in TCP mode')

    def read_bytes_async(self, *args):
        raise NotImplementedError('Async read not available in TCP mode')

    center_freq = fc = property(get_center_freq, set_center_freq)
    sample_rate = rs = property(get_sample_rate, set_sample_rate)
    bandwidth = property(get_bandwidth, set_bandwidth)
    gain = property(get_gain, set_gain)
    freq_correction = property(get_freq_correction, set_freq_correction)



#    This file is part of pyrlsdr.
#    Copyright (C) 2013 by Roger <https://github.com/roger-/pyrtlsdr>
#
#    pyrlsdr is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    pyrlsdr is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with pyrlsdr.  If not, see <http://www.gnu.org/licenses/>.

import os
import warnings
import pkg_resources

try:
    __version__ = pkg_resources.require('pyrtlsdr')[0].version
except: # pragma: no cover
    __version__ = 'unknown'

RTLSDR_CLIENT_MODE = False
if os.environ.get('RTLSDR_CLIENT_MODE', '').lower() in ['true', '1', 'yes']:
    RTLSDR_CLIENT_MODE = True

class ClientModeWarning(UserWarning):
    def __init__(self):
        msg = '\n'.join([
            'Running in "client-only" mode: {varname}="{val}"',
            'Only Tcp communication will be available (RtlSdrTcpClient)',
            'If this is was not intended, set "{varname}" to "false" or remove it',
        ]).format(varname='$RTLSDR_CLIENT_MODE', val=os.environ['RTLSDR_CLIENT_MODE'])
        super(ClientModeWarning, self).__init__(msg)

def warn_client_mode():
    def formatwarning(message, category, filename, lineno, line=None):
        return '{}: {}: \n{}\n'.format(filename, category.__name__, message)
    orig_fmt = warnings.formatwarning
    warnings.formatwarning = formatwarning
    warnings.warn(ClientModeWarning())
    warnings.formatwarning = orig_fmt

if RTLSDR_CLIENT_MODE:
    warn_client_mode()
    from .rtlsdrtcp.client import RtlSdrTcpClient
    librtlsdr = None
    RtlSdr = None
    RtlSdrTcpServer = None
    RtlSdrAio = None
    AIO_AVAILABLE = False
else:
    from .librtlsdr import librtlsdr
    from .rtlsdr import RtlSdr
    from .rtlsdrtcp import RtlSdrTcpServer, RtlSdrTcpClient
    from .helpers import limit_calls, limit_time
    from .rtlsdraio import RtlSdrAio, AIO_AVAILABLE

if AIO_AVAILABLE:
    RtlSdr = RtlSdrAio


__all__  = ['librtlsdr', 'RtlSdr', 'RtlSdrTcpServer', 'RtlSdrTcpClient',
            'limit_calls', 'limit_time']



"""
This module allows client/server communication.

The :class:`~rtlsdr.rtlsdrtcp.server.RtlSdrTcpServer` class is meant to be
connected physically to an SDR dongle and communicate with an instance of
:class:`~rtlsdr.rtlsdrtcp.client.RtlSdrTcpClient`.

The client is intended to function as closely as possible to the base
:class:`~rtlsdr.rtlsdr.RtlSdr` class (as if it had a physical dongle
attatched to it).

Both of these classes have the same arguments as the base
:class:`~rtlsdr.rtlsdr.RtlSdr` class with the addition of ``hostname`` and ``port``.

Examples:
    .. code-block:: python

       server = RtlSdrTcpServer(hostname='192.168.1.100', port=12345)
       server.run_forever()
       # Will listen for clients until Ctrl-C is pressed

    .. code-block:: python

       # On another machine (typically)
       client = RtlSdrTcpClient(hostname='192.168.1.100', port=12345)
       client.center_freq = 2e6
       data = client.read_samples()

Note:
    On platforms where the ``librtlsdr`` library cannot be installed/compiled,
    it is possible to import :class:`~rtlsdr.rtlsdrtcp.RtlSdrTcpClient` only by
    setting the environment variable ``"RTLSDR_CLIENT_MODE"`` to ``"true"``.
    If this is set, no other modules will be available.

*Feature added in v0.2.4*
"""

import os

RTLSDR_CLIENT_MODE = False
if os.environ.get('RTLSDR_CLIENT_MODE', '').lower() in ['true', '1', 'yes']:
    RTLSDR_CLIENT_MODE = True

from .client import RtlSdrTcpClient
try:
    from .server import RtlSdrTcpServer
except ImportError:
    if not RTLSDR_CLIENT_MODE:
        raise



#    This file is part of pyrlsdr.
#    Copyright (C) 2013 by Roger <https://github.com/roger-/pyrtlsdr>
#
#    pyrlsdr is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    pyrlsdr is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with pyrlsdr.  If not, see <http://www.gnu.org/licenses/>.


from __future__ import division
import matplotlib.animation as animation
from matplotlib.mlab import psd
import pylab as pyl
import numpy as np
import sys
from rtlsdr import RtlSdr

# A simple waterfall, spectrum plotter
#
# Controls:
#
# * Scroll mouse-wheel up or down, or press the left or right arrow keys, to
#   change the center frequency (hold shift for finer control).
# * Press "+" and "-" to control gain, and space to enable AGC.
# * Type a frequency (in MHz) and press enter to directly change the center frequency

NFFT = 1024*4
NUM_SAMPLES_PER_SCAN = NFFT*16
NUM_BUFFERED_SWEEPS = 100

# change this to control the number of scans that are combined in a single sweep
# (e.g. 2, 3, 4, etc.) Note that it can slow things down
NUM_SCANS_PER_SWEEP = 1

# these are the increments when scrolling the mouse wheel or pressing '+' or '-'
FREQ_INC_COARSE = 1e6
FREQ_INC_FINE = 0.1e6
GAIN_INC = 5

class Waterfall(object):
    keyboard_buffer = []
    shift_key_down = False
    image_buffer = -100*np.ones((NUM_BUFFERED_SWEEPS,\
                                 NUM_SCANS_PER_SWEEP*NFFT))

    def __init__(self, sdr=None, fig=None):
        self.fig = fig if fig else pyl.figure()
        self.sdr = sdr if sdr else RtlSdr()

        self.init_plot()

    def init_plot(self):
        self.ax = self.fig.add_subplot(1,1,1)
        self.image = self.ax.imshow(self.image_buffer, aspect='auto',\
                                    interpolation='nearest', vmin=-50, vmax=10)
        self.ax.set_xlabel('Current frequency (MHz)')
        self.ax.get_yaxis().set_visible(False)

        self.fig.canvas.mpl_connect('scroll_event', self.on_scroll)
        self.fig.canvas.mpl_connect('key_press_event', self.on_key_press)
        self.fig.canvas.mpl_connect('key_release_event', self.on_key_release)

    def update_plot_labels(self):
        fc = self.sdr.fc
        rs = self.sdr.rs
        freq_range = (fc - rs/2)/1e6, (fc + rs*(NUM_SCANS_PER_SWEEP - 0.5))/1e6

        self.image.set_extent(freq_range + (0, 1))
        self.fig.canvas.draw_idle()

    def on_scroll(self, event):
        if event.button == 'up':
            self.sdr.fc += FREQ_INC_FINE if self.shift_key_down else FREQ_INC_COARSE
            self.update_plot_labels()
        elif event.button == 'down':
            self.sdr.fc -= FREQ_INC_FINE if self.shift_key_down else FREQ_INC_COARSE
            self.update_plot_labels()

    def on_key_press(self, event):
        if event.key == '+':
            self.sdr.gain += GAIN_INC
        elif event.key == '-':
            self.sdr.gain -= GAIN_INC
        elif event.key == ' ':
            self.sdr.gain = 'auto'
        elif event.key == 'shift':
            self.shift_key_down = True
        elif event.key == 'right':
            self.sdr.fc += FREQ_INC_FINE if self.shift_key_down else FREQ_INC_COARSE
            self.update_plot_labels()
        elif event.key == 'left':
            self.sdr.fc -= FREQ_INC_FINE if self.shift_key_down else FREQ_INC_COARSE
            self.update_plot_labels()
        elif event.key == 'enter':
            # see if valid frequency was entered, then change center frequency
            try:
                # join individual key presses into a string
                input = ''.join(self.keyboard_buffer)

                # if we're doing multiple adjacent scans, we need to figure out
                # the appropriate center freq for the leftmost scan
                center_freq = float(input)*1e6 + (self.sdr.rs/2)*(1 - NUM_SCANS_PER_SWEEP)
                self.sdr.fc = center_freq

                self.update_plot_labels()
            except ValueError:
                pass

            self.keyboard_buffer = []
        else:
            self.keyboard_buffer.append(event.key)

    def on_key_release(self, event):
        if event.key == 'shift':
            self.shift_key_down = False

    def update(self, *args):
        # save center freq. since we're gonna be changing it
        start_fc = self.sdr.fc

        # prepare space in buffer
        # TODO: use indexing to avoid recreating buffer each time
        self.image_buffer = np.roll(self.image_buffer, 1, axis=0)

        for scan_num, start_ind in enumerate(range(0, NUM_SCANS_PER_SWEEP*NFFT, NFFT)):
            self.sdr.fc += self.sdr.rs*scan_num

            # estimate PSD for one scan
            samples = self.sdr.read_samples(NUM_SAMPLES_PER_SCAN)
            psd_scan, f = psd(samples, NFFT=NFFT)

            self.image_buffer[0, start_ind: start_ind+NFFT] = 10*np.log10(psd_scan)

        # plot entire sweep
        self.image.set_array(self.image_buffer)

        # restore original center freq.
        self.sdr.fc = start_fc

        return self.image,

    def start(self):
        self.update_plot_labels()
        if sys.platform == 'darwin':
            # Disable blitting. The matplotlib.animation's restore_region()
            # method is only implemented for the Agg-based backends,
            # which the macosx backend is not.
            blit = False
        else:
            blit = True
        ani = animation.FuncAnimation(self.fig, self.update, interval=50,
                blit=blit)

        pyl.show()

        return


def main():
    sdr = RtlSdr()
    wf = Waterfall(sdr)

    # some defaults
    sdr.rs = 2.4e6
    sdr.fc = 100e6
    sdr.gain = 10

    wf.start()

    # cleanup
    sdr.close()


if __name__ == '__main__':
    main()



#! /usr/bin/env python
import sys
import threading
import struct
import traceback
import argparse

PY2 = sys.version_info[0] == 2
if PY2:
    from SocketServer import TCPServer, BaseRequestHandler
else:
    from socketserver import TCPServer, BaseRequestHandler

from rtlsdr import RtlSdr

from .base import (
    CommunicationError,
    RtlSdrTcpBase,
    ClientMessage,
    ServerMessage,
    AckMessage,
    DEFAULT_READ_SIZE,
    API_METHODS,
    API_DESCRIPTORS,
)

class RtlSdrTcpServer(RtlSdr, RtlSdrTcpBase):

    """Server that connects to a physical dongle to allow client connections.
    """

    def __init__(self, device_index=0, test_mode_enabled=False, serial_number=None,
                 hostname='127.0.0.1', port=None):

        RtlSdrTcpBase.__init__(self, device_index, test_mode_enabled,
                               hostname, port)
        RtlSdr.__init__(self, device_index, test_mode_enabled, serial_number)

    def open(self, device_index=0, test_mode_enabled=False, serial_number=None):
        if not self.device_ready:
            return
        super(RtlSdrTcpServer, self).open(device_index, test_mode_enabled, serial_number)

    def run(self):
        """Runs the server thread and returns.  Use this only if you are
        running mainline code afterwards.
        The server must explicitly be stopped by the stop method before exit.

        """
        if self.server_thread is None:
            self.server_thread = ServerThread(self)
        if self.server_thread.running.is_set():
            return
        self.server_thread.start()
        self.server_thread.running.wait()
        e = self.server_thread.exception
        if e is not None:
            print(self.server_thread.exception_tb)
            raise e
        if self.server_thread.stopped.is_set():
            self.server_thread = None
            self.close()

    def run_forever(self):
        """Runs the server and begins a mainloop.
        The loop will exit with Ctrl-C.
        """
        self.run()
        while True:
            try:
                self.server_thread.stopped.wait(1.)
            except KeyboardInterrupt:
                self.close()
                break

    def close(self):
        """Stops the server (if it's running) and closes the connection to the
        dongle.
        """
        if self.server_thread is not None:
            if self.server_thread.running.is_set():
                self.server_thread.stop()
            self.server_thread = None
        super(RtlSdrTcpServer, self).close()

    def read_bytes(self, num_bytes=DEFAULT_READ_SIZE):
        """Return a packed string of bytes read along with the struct_fmt.
        """
        fmt_str = '%dB' % (num_bytes)
        buffer = super(RtlSdrTcpServer, self).read_bytes(num_bytes)
        s = struct.pack(fmt_str, *buffer)
        return {'struct_fmt':fmt_str, 'data':s}

    def read_samples(self, num_samples=DEFAULT_READ_SIZE):
        """This overrides the base implementation so that the raw data is sent.
        It will be unpacked to I/Q samples on the client side.
        """
        num_samples = 2*num_samples
        return self.read_bytes(num_samples)

class ServerThread(threading.Thread):
    def __init__(self, rtl_sdr):
        super(ServerThread, self).__init__()
        self.rtl_sdr = rtl_sdr
        self.running = threading.Event()
        self.stopped = threading.Event()
        self.exception = None

    def run(self):
        try:
            self.server = Server(self.rtl_sdr)
        except Exception as e:
            self.exception = e
            self.exception_tb = traceback.format_exc()
            self.running.set()
            self.stopped.set()
            return
        rtl_sdr = self.rtl_sdr
        rtl_sdr.device_ready = True
        rtl_sdr.open(rtl_sdr.device_index, rtl_sdr.test_mode_enabled)
        self.running.set()
        self.server.serve_forever()
        self.running.clear()
        rtl_sdr.device_ready = False
        self.stopped.set()

    def stop(self):
        running = getattr(self, 'running', None)
        if running is None or not running.is_set():
            return
        if not hasattr(self, 'server'):
            return
        self.server.shutdown()
        self.server.server_close()
        self.stopped.wait()


class Server(TCPServer):
    REQUEST_RECV_SIZE = 1024

    def __init__(self, rtl_sdr):
        self.rtl_sdr = rtl_sdr
        server_addr = (rtl_sdr.hostname, rtl_sdr.port)
        TCPServer.__init__(self, server_addr, RequestHandler)
        self.handlers = set()

    def server_close(self):
        if not hasattr(self, 'handlers'):
            return
        for h in self.handlers:
            h.close()

class RequestHandler(BaseRequestHandler):
    def setup(self):
        self.finished = False
        self.server.handlers.add(self)

    def handle(self, rx_message=None):
        if rx_message is None:
            rx_message = ClientMessage.from_remote(self.request)
        msg_type = rx_message.header.get('type')
        if msg_type == 'method':
            r = self.handle_method_call(rx_message)
        elif msg_type == 'prop_set':
            r = self.handle_prop_set(rx_message)
        elif msg_type == 'prop_get':
            r = self.handle_prop_get(rx_message)
        else:
            r = False
        if r is False:
            nak = AckMessage(ok=False)
            nak.send_message(self.request)

    def finish(self):
        self.server.handlers.discard(self)

    def close(self):
        self.finished = True

    def handle_method_call(self, rx_message):
        rtl_sdr = self.server.rtl_sdr
        method_name = rx_message.header.get('name')
        arg = rx_message.data
        if method_name not in API_METHODS:
            raise CommunicationError('method %s not allowed' % (method_name))
        try:
            m = getattr(rtl_sdr, method_name)
        except AttributeError:
            msg = 'sdr has no attribute "%s"' % (method_name)
            raise CommunicationError(msg)
        if arg is not None:
            resp = m(arg)
        else:
            resp = m()
        tx_message = ServerMessage(client_message=rx_message, data=resp)
        tx_message.send_message(self.request)

    def handle_prop_set(self, rx_message):
        rtl_sdr = self.server.rtl_sdr
        prop_name = rx_message.header.get('name')
        value = rx_message.data
        if prop_name not in API_DESCRIPTORS:
            raise CommunicationError('property %s not allowed' % (prop_name))
        setattr(rtl_sdr, prop_name, value)
        tx_message = ServerMessage(client_message=rx_message)
        tx_message.send_message(self.request)

    def handle_prop_get(self, rx_message):
        prop_name = rx_message.header.get('name')
        if prop_name not in API_DESCRIPTORS:
            raise CommunicationError('property %s not allowed' % (prop_name))
        rtl_sdr = self.server.rtl_sdr
        value = getattr(rtl_sdr, prop_name)
        tx_message = ServerMessage(client_message=rx_message, data=value)
        tx_message.send_message(self.request)


def run_server():
    """Convenience function to run the server from the command line
    with options for hostname, port and device index.
    """
    p = argparse.ArgumentParser()
    p.add_argument(
        '-a', '--address',
        dest='hostname',
        metavar='address',
        default='127.0.0.1',
        help='Listen address (default is "127.0.0.1")')
    p.add_argument(
        '-p', '--port',
        dest='port',
        type=int,
        default=1235,
        help='Port to listen on (default is 1235)')
    p.add_argument(
        '-d', '--device-index',
        dest='device_index',
        type=int,
        default=0)
    args, remaining = p.parse_known_args()
    o = vars(args)
    server = RtlSdrTcpServer(**o)
    server.run_forever()

if __name__ == '__main__':
    run_server()



#    This file is part of pyrlsdr.
#    Copyright (C) 2013 by Roger <https://github.com/roger-/pyrtlsdr>
#
#    pyrlsdr is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    pyrlsdr is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with pyrlsdr.  If not, see <http://www.gnu.org/licenses/>.


import os
import sys
import re
from setuptools import setup, find_packages

PACKAGE_NAME = 'pyrtlsdr'
VERSION = '0.2.91'

#HERE = os.path.abspath(os.path.dirname(__file__))
#README = open(os.path.join(HERE, 'README.md')).read()

def convert_readme():
    from m2r import parse_from_file
    rst = parse_from_file('README.md')
    with open('README.rst', 'w') as f:
        f.write(rst)
    return rst

def read_rst():
    try:
        with open('README.rst', 'r') as f:
            rst = f.read()
    except IOError:
        rst = None
    return rst

if {'sdist', 'bdist_wheel'} & set(sys.argv):
    long_description = convert_readme()
else:
    long_description = read_rst()

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    author='roger',
    url='https://github.com/roger-/pyrtlsdr',
    description='A Python wrapper for librtlsdr (a driver for Realtek RTL2832U based SDR\'s)',
    long_description=long_description,
    classifiers=['Development Status :: 4 - Beta',
                 'Environment :: Console',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: GNU General Public License (GPL)',
                 'Natural Language :: English',
                 'Operating System :: OS Independent',
                 'Programming Language :: Python :: 2',
                 'Programming Language :: Python :: 2.7',
                 'Programming Language :: Python :: 3',
                 'Programming Language :: Python :: 3.4',
                 'Programming Language :: Python :: 3.5',
                 'Programming Language :: Python :: 3.6',
                 'Programming Language :: Python :: 3.7',
                 'Topic :: Utilities'],
    license='GPLv3',
    keywords='radio librtlsdr rtlsdr sdr',
    setup_requires=['m2r'],
    platforms=['any'],
    packages=find_packages(exclude=['tests*']))



#    This file is part of pyrlsdr.
#    Copyright (C) 2013 by Roger <https://github.com/roger-/pyrtlsdr>
#
#    pyrlsdr is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    pyrlsdr is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with pyrlsdr.  If not, see <http://www.gnu.org/licenses/>.


from __future__ import division
from __future__ import print_function
from rtlsdr import *

def main():

    @limit_calls(2)
    def test_callback(samples, rtlsdr_obj):
        print('  in callback')
        print('  signal mean:', sum(samples)/len(samples))

    sdr = RtlSdr()

    print('Configuring SDR...')
    sdr.rs = 2.4e6
    sdr.fc = 100e6
    sdr.gain = 10
    print('  sample rate: %0.6f MHz' % (sdr.rs/1e6))
    print('  center frequency %0.6f MHz' % (sdr.fc/1e6))
    print('  gain: %d dB' % sdr.gain)

    print('Reading samples...')
    samples = sdr.read_samples(256*1024)
    print('  signal mean:', sum(samples)/len(samples))

    print('Testing callback...')
    sdr.read_samples_async(test_callback, 256*1024)

    try:
        import pylab as mpl

        print('Testing spectrum plotting...')
        mpl.figure()
        mpl.psd(samples, NFFT=1024, Fc=sdr.fc/1e6, Fs=sdr.rs/1e6)

        mpl.show()
    except:
        # matplotlib not installed/working
        pass

    print('Done\n')
    sdr.close()

if __name__ == '__main__':
    main()



#    This file is part of pyrlsdr.
#    Copyright (C) 2013 by Roger <https://github.com/roger-/pyrtlsdr>
#
#    pyrlsdr is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    pyrlsdr is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with pyrlsdr.  If not, see <http://www.gnu.org/licenses/>.


from __future__ import division, print_function
from ctypes import *
from .librtlsdr import (
    librtlsdr,
    p_rtlsdr_dev,
    rtlsdr_read_async_cb_t,
    tuner_bandwidth_supported,
    tuner_set_bandwidth_supported,
)
try:                from itertools import izip
except ImportError: izip = zip
import sys

PY3 = sys.version_info.major >= 3
if PY3:
    basestring = str

# see if NumPy is available
has_numpy = True
try:
    import numpy as np
except ImportError:
    has_numpy = False


class BaseRtlSdr(object):
    """Core interface for most API functionality

    Arguments:
        device_index (:obj:`int`, optional): The device index to use if there are
            multiple dongles attached.  If only one is being used,
            the default value (0) will be used.
        test_mode_enabled (:obj:`bool`, optional): If True, enables a special
            test mode, which will return the value of an internal RTL2832
            8-bit counter with calls to :meth:`read_bytes`.
        serial_number (:obj:`str`, optional): If not None, the device will be searched
            for by the given serial_number by :meth:`get_device_index_by_serial`
            and the ``device_index`` returned will be used automatically.

    Attributes:
        DEFAULT_GAIN: Default :attr:`gain` value used on initialization: ``'auto'``
        DEFAULT_FC (float): Default :attr:`center_freq` value used on
            initialization: ``80e6`` (80 Mhz)
        DEFAULT_RS (float): Default :attr:`sample_rate` value used on
            initialization: ``1.024e6`` (1024 Msps)
        DEFAULT_READ_SIZE (int): Default number of samples or bytes to read
            if no arguments are supplied for :meth:`read_bytes`
            or :meth:`read_samples`.  Default value is ``1024``
        gain_values (list(int)): The valid gain parameters supported by the device
            (in tenths of dB). These are stored as returned by ``librtlsdr``.
        valid_gains_db (list(float)): The valid gains in dB

    """
    # some default values for various parameters
    DEFAULT_GAIN = 'auto'
    DEFAULT_FC = 80e6
    DEFAULT_RS = 1.024e6
    DEFAULT_READ_SIZE = 1024

    CRYSTAL_FREQ = 28800000

    gain_values = []
    valid_gains_db = []
    buffer = []
    num_bytes_read = c_int32(0)
    device_opened = False

    @staticmethod
    def get_device_index_by_serial(serial):
        """Retrieves the device index for a device matching the given serial number

        Arguments:
            serial (str): The serial number to search for

        Returns:
            int: The device_index as reported by ``librtlsdr``

        Notes:
            Most devices by default have the same serial number: `'0000001'`.
            This can be set to a custom value by using the `rtl\_eeprom`_ utility
            packaged with ``librtlsdr``.

        .. _rtl\_eeprom: http://manpages.ubuntu.com/manpages/trusty/man1/rtl_eeprom.1.html

        """
        if PY3 and isinstance(serial, str):
            serial = bytes(serial, 'UTF-8')

        result = librtlsdr.rtlsdr_get_index_by_serial(serial)
        if result < 0:
            raise IOError('Error code %d when searching device by serial' % (result))

        return result

    @staticmethod
    def get_device_serial_addresses():
        """Get serial numbers for all attached devices

        Returns:
            list(str): A ``list`` of all detected serial numbers (``str``)

        """
        def get_serial(device_index):
            bfr = (c_ubyte * 256)()
            r = librtlsdr.rtlsdr_get_device_usb_strings(device_index, None, None, bfr)
            if r != 0:
                raise IOError(
                    'Error code %d when reading USB strings (device %d)' % (r, device_index)
                )
            return ''.join((chr(b) for b in bfr if b > 0))

        num_devices = librtlsdr.rtlsdr_get_device_count()
        return [get_serial(i) for i in range(num_devices)]

    def __init__(self, device_index=0, test_mode_enabled=False, serial_number=None):
        self.open(device_index, test_mode_enabled, serial_number)

    def open(self, device_index=0, test_mode_enabled=False, serial_number=None):
        """Connect to the device through the underlying wrapper library

        Initializes communication with the device and retrieves information
        from it with a call to :meth:`init_device_values`.

        Arguments:
            device_index (:obj:`int`, optional): The device index to use if there are
                multiple dongles attached.  If only one is being used,
                the default value (0) will be used.
            test_mode_enabled (:obj:`bool`, optional): If True, enables a special
                test mode, which will return the value of an internal RTL2832
                8-bit counter with calls to :meth:`read_bytes`.
            serial_number (:obj:`str`, optional): If not None, the device will be searched
                for by the given serial_number by :meth:`get_device_index_by_serial`
                and the ``device_index`` returned will be used automatically.

        Notes:
            The arguments used here are passed directly from object
            initialization.

        Raises:
            IOError: If communication with the device could not be established.

        """

        if serial_number is not None:
            device_index = self.get_device_index_by_serial(serial_number)

        # this is the pointer to the device structure used by all librtlsdr
        # functions
        self.dev_p = p_rtlsdr_dev(None)

        # initialize device
        result = librtlsdr.rtlsdr_open(self.dev_p, device_index)
        if result < 0:
            raise IOError('Error code %d when opening SDR (device index = %d)'\
                          % (result, device_index))

        # enable test mode if necessary
        result = librtlsdr.rtlsdr_set_testmode(self.dev_p, int(test_mode_enabled))
        if result < 0:
            raise IOError('Error code %d when setting test mode'\
                          % (result))

        # reset buffers
        result = librtlsdr.rtlsdr_reset_buffer(self.dev_p)
        if result < 0:
            raise IOError('Error code %d when resetting buffer (device index = %d)'\
                          % (result, device_index))

        self.device_opened = True
        self.init_device_values()

    def init_device_values(self):
        """Retrieves information from the device

        This method acquires the values for :attr:`gain_values`. Also sets the
        device to the default :attr:`center frequency <DEFAULT_FC>`, the
        :attr:`sample rate <DEFAULT_RS>` and :attr:`gain <DEFAULT_GAIN>`
        """
        self.gain_values = self.get_gains()
        self.valid_gains_db = [val/10 for val in self.gain_values]

        # set default state
        self.set_sample_rate(self.DEFAULT_RS)
        self.set_center_freq(self.DEFAULT_FC)
        self.set_gain(self.DEFAULT_GAIN)

    def close(self):
        if not self.device_opened:
            return

        librtlsdr.rtlsdr_close(self.dev_p)
        self.device_opened = False

    def __del__(self):
        self.close()

    def set_center_freq(self, freq):

        freq = int(freq)

        result = librtlsdr.rtlsdr_set_center_freq(self.dev_p, freq)
        if result < 0:
            self.close()
            raise IOError('Error code %d when setting center freq. to %d Hz'\
                          % (result, freq))

        return

    def get_center_freq(self):

        result = librtlsdr.rtlsdr_get_center_freq(self.dev_p)
        if result < 0:
            self.close()
            raise IOError('Error code %d when getting center freq.'\
                          % (result))

        # FIXME: the E4000 rounds to kHz, this may not be true for other tuners
        reported_center_freq = result
        center_freq = round(reported_center_freq, -3)

        return center_freq

    def set_freq_correction(self, err_ppm):

        freq = int(err_ppm)

        result = librtlsdr.rtlsdr_set_freq_correction(self.dev_p, err_ppm)
        if result < 0:
            self.close()
            raise IOError('Error code %d when setting freq. offset to %d ppm'\
                          % (result, err_ppm))

        return

    def get_freq_correction(self):

        result = librtlsdr.rtlsdr_get_freq_correction(self.dev_p)
        if result < 0:
            self.close()
            raise IOError('Error code %d when getting freq. offset in ppm.'\
                          % (result))
        return result

    def set_sample_rate(self, rate):

        rate = int(rate)

        result = librtlsdr.rtlsdr_set_sample_rate(self.dev_p, rate)
        if result < 0:
            self.close()
            raise IOError('Error code %d when setting sample rate to %d Hz'\
                          % (result, rate))

        return

    def get_sample_rate(self):

        result = librtlsdr.rtlsdr_get_sample_rate(self.dev_p)
        if result < 0:
            self.close()
            raise IOError('Error code %d when getting sample rate'\
                          % (result))

        # figure out actual sample rate, taken directly from librtlsdr
        reported_sample_rate = result
        rsamp_ratio = (self.CRYSTAL_FREQ * pow(2, 22)) // reported_sample_rate
        rsamp_ratio &= ~3
        real_rate = (self.CRYSTAL_FREQ * pow(2, 22)) / rsamp_ratio;

        return real_rate

    def set_bandwidth(self, bw):

        bw = int(bw)
        if tuner_bandwidth_supported:
            apply_bw = c_int(1)
            applied_bw = c_uint32(bw)
            bw = c_uint32(bw)
            result = librtlsdr.rtlsdr_set_and_get_tuner_bandwidth(
                self.dev_p, bw, byref(applied_bw), apply_bw)
            self._bandwidth = applied_bw.value
        elif tuner_set_bandwidth_supported:
            bw = int(bw)
            result = librtlsdr.rtlsdr_set_tuner_bandwidth(self.dev_p, bw)
            self._bandwidth = bw
        else:
            raise IOError('set_tuner_bandwidth not supported in this version of librtlsdr')

        if result != 0:
            self.close()
            raise IOError('Error code %d when setting tuner bandwidth to %d Hz'\
                          % (result, bw))

        return

    def get_bandwidth(self):

        return getattr(self, '_bandwidth', 0)

    def set_gain(self, gain):
        if isinstance(gain, basestring) and gain == 'auto':
            # disable manual gain -> enable AGC
            self.set_manual_gain_enabled(False)

            return

        # find supported gain nearest to one requested
        errors = [abs(10*gain - g) for g in self.gain_values]
        nearest_gain_ind = errors.index(min(errors))

        # disable AGC
        self.set_manual_gain_enabled(True)

        result = librtlsdr.rtlsdr_set_tuner_gain(self.dev_p,
                                                 self.gain_values[nearest_gain_ind])
        if result < 0:
            self.close()
            raise IOError('Error code %d when setting gain to %d'\
                          % (result, gain))

        return

    def get_gain(self):

        result = librtlsdr.rtlsdr_get_tuner_gain(self.dev_p)
        if 0 and result == 0:
            self.close()
            raise IOError('Error when getting gain')

        return result/10

    def get_gains(self):
        """Get all supported gain values from driver

        Returns:
            list(int): Gains in tenths of a dB
        """
        buffer = (c_int *50)()
        result = librtlsdr.rtlsdr_get_tuner_gains(self.dev_p, buffer)
        if result == 0:
            self.close()
            raise IOError('Error when getting gains')

        gains = []
        for i in range(result):
            gains.append(buffer[i])

        return gains

    def set_manual_gain_enabled(self, enabled):
        """Enable or disable manual gain control of tuner.

        Arguments:
            enabled (bool):

        Notes:
            If ``enabled`` is False, then AGC should also be used by calling
            :meth:`set_agc_mode`. It is recommended to use :meth:`set_gain`
            instead of calling this method directly.
        """
        result = librtlsdr.rtlsdr_set_tuner_gain_mode(self.dev_p, int(enabled))
        if result < 0:
            raise IOError('Error code %d when setting gain mode'\
                          % (result))

        return

    def set_agc_mode(self, enabled):
        """Enable RTL2832 AGC

        Arguments:
            enabled (bool):
        """
        result = librtlsdr.rtlsdr_set_agc_mode(self.dev_p, int(enabled))
        if result < 0:
            raise IOError('Error code %d when setting AGC mode'\
                          % (result))

        return result

    def set_direct_sampling(self, direct):
        """Enable direct sampling.

        Arguments:
            direct: If False or 0, disable direct sampling.  If 'i' or 1,
                use ADC I input.  If 'q' or 2, use ADC Q input.
        """

        # convert parameter
        if isinstance(direct, basestring):
            if direct.lower() == 'i':
                direct = 1
            elif direct.lower() == 'q':
                direct = 2
            else:
                raise SyntaxError('invalid value "%s"' % direct)

        # make sure False works as an option
        if not direct:
            direct = 0

        result = librtlsdr.rtlsdr_set_direct_sampling(self.dev_p, direct)
        if result < 0:
            raise IOError('Error code %d when setting AGC mode'\
                          % (result))

        return result

    def get_tuner_type(self):
        """Get the tuner type.

        Returns:
            int:
                The tuner type as reported by the driver.
                See the `tuner enum definition`_ for more information.

        .. _tuner enum definition: https://github.com/librtlsdr/librtlsdr/blob/c7d970ac5b70e897501909a48b2b32d4bfb16979/include/rtl-sdr.h#L185-L201
        """
        result = librtlsdr.rtlsdr_get_tuner_type(self.dev_p)
        if result < 0:
            raise IOError('Error code %d when getting tuner type'\
                          % (result))

        return result

    def read_bytes(self, num_bytes=DEFAULT_READ_SIZE):
        """Read specified number of bytes from tuner.

        Does not attempt to unpack complex samples (see :meth:`read_samples`),
        and data may be unsafe as buffer is reused.

        Arguments:
            num_bytes (:obj:`int`, optional): The number of bytes to read.
                Defaults to :attr:`DEFAULT_READ_SIZE`.

        Returns:
            ctypes.Array[c_ubyte]:
                A buffer of len(num_bytes) containing the raw samples read.
        """
        # FIXME: libsdrrtl may not be able to read an arbitrary number of bytes

        num_bytes = int(num_bytes)

        # create buffer, as necessary
        if len(self.buffer) != num_bytes:
            array_type = (c_ubyte*num_bytes)
            self.buffer = array_type()

        result = librtlsdr.rtlsdr_read_sync(self.dev_p, self.buffer, num_bytes,\
                                            byref(self.num_bytes_read))
        if result < 0:
            self.close()
            raise IOError('Error code %d when reading %d bytes'\
                          % (result, num_bytes))

        if self.num_bytes_read.value != num_bytes:
            self.close()
            raise IOError('Short read, requested %d bytes, received %d'\
                          % (num_bytes, self.num_bytes_read.value))

        return self.buffer

    def read_samples(self, num_samples=DEFAULT_READ_SIZE):
        """Read specified number of complex samples from tuner.

        Real and imaginary parts are normalized to be in the range [-1, 1].
        Data is safe after this call (will not get overwritten by another one).

        Arguments:
            num_samples (:obj:`int`, optional): Number of samples to read.
                Defaults to :attr:`DEFAULT_READ_SIZE`.

        Returns:
            The samples read as either a :class:`list` or :class:`numpy.ndarray`
            (if available).
        """
        num_bytes = 2*num_samples

        raw_data = self.read_bytes(num_bytes)
        iq = self.packed_bytes_to_iq(raw_data)

        return iq

    def packed_bytes_to_iq(self, bytes):
        """Unpack a sequence of bytes to a sequence of normalized complex numbers

        This is called automatically by :meth:`read_samples`.

        Returns:
            The unpacked iq values as either a :class:`list` or
            :class:`numpy.ndarray` (if available).
        """
        if has_numpy:
            # use NumPy array
            data = np.ctypeslib.as_array(bytes)
            iq = data.astype(np.float64).view(np.complex128)
            iq /= 127.5
            iq -= (1 + 1j)
        else:
            # use normal list
            iq = [complex(i/(255/2) - 1, q/(255/2) - 1) for i, q in izip(bytes[::2], bytes[1::2])]

        return iq

    center_freq = fc = property(get_center_freq, set_center_freq,
        doc="""int: Get/Set the center frequency of the device (in Hz)""")
    sample_rate = rs = property(get_sample_rate, set_sample_rate,
        doc="""int: Get/Set the sample rate of the tuner (in Hz)""")
    gain = property(get_gain, set_gain,
        doc="""float or str: Get/Set gain of the tuner (in dB)

        Notes:
            If set to 'auto', AGC mode is enabled; otherwise gain is in dB.
            The actual gain used is rounded to the nearest value supported by
            the device (see the values in :attr:`valid_gains_db`).
        """)
    freq_correction = property(get_freq_correction, set_freq_correction,
        doc="""int: Get/Set frequency offset of the tuner (in PPM)""")
    bandwidth = property(get_bandwidth, set_bandwidth,
        doc="""int: Get/Set bandwidth value (in Hz)

        Set value to 0 (default) for automatic bandwidth selection.

        Notes:
            This value is stored locally and may not reflect the real tuner bandwidth

        """)


# This adds async read support to base class BaseRtlSdr (don't use that one)
class RtlSdr(BaseRtlSdr):
    """This adds async read support to :class:`BaseRtlSdr`
    """
    DEFAULT_ASYNC_BUF_NUMBER = 0 # librtlsdr will use the default (15)
    DEFAULT_READ_SIZE = 1024

    read_async_canceling = False

    def read_bytes_async(self, callback, num_bytes=DEFAULT_READ_SIZE, context=None):
        """Continuously read bytes from tuner

        Arguments:
            callback: A function or method that will be called with the result.
                See :meth:`_bytes_converter_callback` for the signature.
            num_bytes (int): Number of bytes to read for each callback.
                Defaults to :attr:`DEFAULT_READ_SIZE`.
            context (Optional): Object to be passed as an argument to the callback.
                If not supplied or None, the :class:`RtlSdr` instance
                will be used.

        Notes:
            As with :meth:`~BaseRtlSdr.read_bytes`, the data passed to the
            callback may by overwritten.
        """
        num_bytes = int(num_bytes)

        # we don't call the provided callback directly, but add a layer inbetween
        # to convert the raw buffer to a safer type

        # save requested callback
        self._callback_bytes = callback

        # convert Python callback function to a librtlsdr callback
        rtlsdr_callback = rtlsdr_read_async_cb_t(self._bytes_converter_callback)

        # use this object as context if none provided
        if not context:
            context = self

        self.read_async_canceling = False
        result = librtlsdr.rtlsdr_read_async(self.dev_p, rtlsdr_callback,\
                    context, self.DEFAULT_ASYNC_BUF_NUMBER, num_bytes)
        if result < 0:
            self.close()
            raise IOError('Error code %d when requesting %d bytes'\
                          % (result, num_bytes))

        self.read_async_canceling = False

        return

    def _bytes_converter_callback(self, raw_buffer, num_bytes, context):
        """Converts the raw buffer used in ``rtlsdr_read_async`` to a usable type

        This method is used internally by :meth:`read_bytes_async` to convert
        the raw data from ``rtlsdr_read_async`` into a memory-safe array.

        The callback given in :meth:`read_bytes_async` will then be called
        with the signature::

            callback(values, context)

        Arguments:
            raw_buffer: Buffer of type ``unsigned char``
            num_bytes (int): Length of ``raw_buffer``
            context: User-defined value passed to ``rtlsdr_read_async``.
                In most cases, will be a reference to the :class:`RtlSdr` instance

        Notes:
            This method is not meant to be called directly or
            overridden by subclasses.

        """
        array_type = (c_ubyte*num_bytes)
        values = cast(raw_buffer, POINTER(array_type)).contents

        # skip callback if cancel_read_async() called
        if self.read_async_canceling:
            return

        self._callback_bytes(values, context)

    def read_samples_async(self, callback, num_samples=DEFAULT_READ_SIZE, context=None):
        """Continuously read 'samples' from the tuner

        This is a combination of :meth:`read_samples` and :meth:`read_bytes_async`

        Arguments:
            callback: A function or method that will be called with the result.
                See :meth:`_samples_converter_callback` for the signature.
            num_samples (int): The number of samples read into each callback.
                Defaults to :attr:`DEFAULT_READ_SIZE`.
            context (Optional): Object to be passed as an argument to the callback.
                If not supplied or None, the :class:`RtlSdr` instance
                will be used.
        """

        num_bytes = 2*num_samples

        self._callback_samples = callback
        self.read_bytes_async(self._samples_converter_callback, num_bytes, context)

        return

    def _samples_converter_callback(self, buffer, context):
        """Converts the raw buffer used in ``rtlsdr_read_async`` to a usable type

        This method is used internally by :meth:`read_samples_async` to convert
        the data into a sequence of complex numbers.

        The callback given in :meth:`read_samples_async` will then be called
        with the signature::

            callback(samples, context)

        Arguments:
            buffer: Buffer of type ``unsigned char``
            context: User-defined value passed to ``rtlsdr_read_async``.
                In most cases, will be a reference to the :class:`RtlSdr` instance

        Notes:
            This method is not meant to be called directly or
            overridden by subclasses.

        """
        iq = self.packed_bytes_to_iq(buffer)

        self._callback_samples(iq, context)

    def cancel_read_async(self):
        """Cancel async read.
        This should be called eventually when using async reads
        (:meth:`read_bytes_async` or :meth:`read_samples_async`),
        or callbacks will never stop.

        See Also:
            :func:`~rtlsdr.helpers.limit_time` and
            :func:`~rtlsdr.helpers.limit_calls`
        """

        result = librtlsdr.rtlsdr_cancel_async(self.dev_p)
        # sometimes we get additional callbacks after canceling an async read,
        # in this case we don't raise exceptions
        if result < 0 and not self.read_async_canceling:
            self.close()
            raise IOError('Error code %d when canceling async read'\
                          % (result))

        self.read_async_canceling = True
